package APKPM::Store;

use base qw(Gearman::Driver::Worker);
use Moose;
use Time::HiRes;
use Data::Dump qw(dump);
use DBD::Pg;
use Redis;

with 'APKPM::Gearman';

sub prefix { 'Store_' }

sub process_name {
	my ( $self, $orig, $job_name ) = @_;
	warn "# process_name $orig $job_name\n";
	return "$orig ($job_name)";
}

sub dbh {
	DBI->connect_cached('DBI:Pg:dbname=apkpm','','', {
		RaiseError => 1,
		AutoCommit => 1,
	});
}

our $redis;
sub redis { $redis ||= Redis->new };

sub pg_insert {
	my ( $self, $table, $h ) = @_;

	my @c;

	if ( my $cols = $self->redis->get("pg.$table") ) {
		@c = split(/\s+/,$cols);
	} else {
		my $sth = $self->dbh->prepare( "select * from $table limit 1" );
		$sth->execute;
		@c = @{ $sth->{NAME_lc} };
		$self->redis->set( "pg.$table" => join(' ',@c) );
		$self->redis->expire( "pg.$table" => 15 * 60 ); # refresh every 15 min
	}

	my $sql = "INSERT INTO $table (" . join(',',@c) . ') values (' . join(',', map { '?' } 0 .. $#c) . ')';
	warn $sql;
	my $sth = $self->dbh->prepare($sql);

	my $h_lc;
	$h_lc->{ lc $_ } = $h->{$_} foreach keys %$h;

	if ( my $username = $h->{username} ) {
		my $key = join('.', 'table', $table, $username);
		$self->redis->set( $key => $self->e_json($h) );
		$self->redis->expire( $key => 15 * 60 ); # 15 min timeout
	}

	$h_lc->{h} = $self->to_hstore( $h_lc->{h} ) if exists $h_lc->{h};

	$sth->execute( map { $h_lc->{$_} } @c );
}

sub insert : Job : Decode(d_json) : MinProcesses(0) {
	my ( $self, $job, $workload ) = @_;
	my $table = delete $workload->{_table} || die "no _table";
	$self->pg_insert($table => $workload);
}

sub sql : Job : Encode(e_json) : MinProcesses(1) {
	my ( $self, $job, $workload ) = @_;

	my $sth = $self->dbh->prepare($workload);
	my $rows = eval { $sth->execute };
	return { error => $@ } if $@;

	warn "# $rows rows get_username_table $workload\n";

	$rows = $sth->fetchall_arrayref;
	my @columns = @{ $sth->{NAME} };

	# decode hash column
	my $hash_col;
	foreach ( 0 .. $#columns ) {
		$hash_col = $_ if $columns[$_] eq 'h';
	}
	if ( defined $hash_col ) {
		map {
			my $hash = $_->[$hash_col];
			$hash =~ s/\@/\\\@/g;
			$_->[$hash_col] = eval "{ $hash }";
			$_->[$hash_col] = "ERROR: $@ for $hash" if $@;
		} @$rows
	}

	return {
		columns => \@columns,
		rows => $rows,
		hash_col => $hash_col,
	};
}

1;
